import { Component, OnInit } from '@angular/core';
import { ReviewManagementService } from 'app/admin/review-management/review-management.service';
import { Review } from 'app/admin/review-management/review.model';
import { ReviewRequest } from 'app/admin/review-management/reviewRequest.model';
import { UserManagementService } from 'app/admin/user-management/service/user-management.service';
import { User } from 'app/admin/user-management/user-management.model';
import { Account } from 'app/core/auth/account.model';
import { AccountService } from 'app/core/auth/account.service';
import { SearchService } from 'app/search/service/search-service';
import { Exercise, searchResultToExercise } from 'app/shared/model/exercise.model';
import { SearchResultsDTO } from 'app/shared/model/search/search-results-dto.model';

@Component({
  selector: 'jhi-peer-reviewing',
  templateUrl: './peer-reviewing.component.html',
  styleUrls: ['./peer-reviewing.component.scss'],
})
export class PeerReviewingComponent implements OnInit {
  isChecked = false;
  currentAccount: Account | null = null;
  user: User | null = null;
  results: Exercise[] = [];
  selectedExercise: Exercise | undefined;
  reviews: Review[] = [];
  selectedReview = new Review('', [''], [''], ['']);
  options = ['Review Requested', 'Review In Progress', 'Improvements Requested', 'Review Rejected', 'Review Completed'];

  constructor(
    private userService: UserManagementService,
    private accountService: AccountService,
    private searchService: SearchService,
    private reviewManagementService: ReviewManagementService
  ) {}

  ngOnInit(): void {
    this.accountService.identity().subscribe(account => (this.currentAccount = account));
    this.getUserFromLogin();
    this.searchForExercises();
    this.loadAll();
  }

  getUserFromLogin(): void {
    if (this.currentAccount != null && this.currentAccount.login != null) {
      this.userService.find(this.currentAccount?.login).subscribe(val => {
        this.user = val;
      });
    }
  }

  loadAll(): void {
    this.reviewManagementService.getAllByUser().subscribe(reviews => {
      this.reviews = reviews;
    });
  }

  setReviewerStatusForLogin(): void {
    if (this.user != null && this.currentAccount != null && this.user.login == this.currentAccount.login) {
      this.userService.updateReviewerStatus(!this.user.reviewingEnabled).subscribe();
    }
  }

  searchForExercises(): void {
    this.searchService.getResultsForLoggedInUser().subscribe(
      (data: SearchResultsDTO) => {
        const searchResults = data.searchResult.map(searchResultToExercise);
        this.results = this.results
          .concat(searchResults)
          .filter((value, index, self) => self.findIndex(t => t.title === value.title) === index);
      },
      () => alert('Search failed')
    );
  }

  shareExercise(exercise: Exercise) {
    this.selectedExercise = exercise;
  }

  requestReview(): void {
    if (this.currentAccount) {
      this.searchService
        .requestReview(
          new ReviewRequest(
            this.selectedExercise!.title,
            this.selectedExercise!.originalResult.exerciseId,
            [this.currentAccount.email],
            this.selectedExercise!.gitlabURL
          )
        )
        .subscribe(() => this.loadAll());
    }
  }

  resourceImproved(review: Review): void {
    review.status![0] = 'Resource Improved';
    this.reviewManagementService.notifyReviewer(review).subscribe(() => this.loadAll());
  }

  updateReview(): void {
    this.reviewManagementService.updateByUser(this.selectedReview).subscribe(() => this.loadAll());
  }

  deleteReview(review: Review) {
    review.status![0] = 'DELETION REQUESTED';
    this.reviewManagementService.updateByUser(review).subscribe(() => this.loadAll());
  }
}
