import { browser, element, by, ExpectedConditions as ec } from 'protractor';

import { NavBarPage, SignInPage } from '../page-objects/jhi-page-objects';

const expect = chai.expect;

describe('account', () => {
  let navBarPage: NavBarPage;
  let signInPage: SignInPage;
  const username = process.env.E2E_USERNAME ?? 'admin';
  const password = process.env.E2E_PASSWORD ?? 'search@admin';

  before(async () => {
    await browser.get('/');
    await browser.sleep(2000);
    navBarPage = new NavBarPage(true);
  });

  it('should fail to login with bad password', async () => {
    browser.get('/login');
    await browser.sleep(2000);
    signInPage = navBarPage.getSignInPage();
    await signInPage.autoSignInUsing(username, 'foo');
    const expect2 = 'login.messages.error.authentication';
    const value2 = await element(by.id('failedLogin')).getAttribute('jhiTranslate');
    expect(value2).to.eq(expect2);
  });

  it('should login successfully with admin account', async () => {
    browser.get('/login');
    await browser.sleep(2000);
    signInPage = navBarPage.getSignInPage();
    await signInPage.autoSignInUsing(username, password);
    const expect2 = 'home.logged.message';
    await browser.wait(ec.visibilityOf(element(by.id('home-logged-message'))));
    const value2 = await element(by.id('home-logged-message')).getAttribute('jhiTranslate');
    expect(value2).to.eq(expect2);
  });

  it('should navigate to 404 not found error page on non existing route and show user own navbar if valid authentication exists', async () => {
    // don't sign out and refresh page with non existing route
    await browser.get('/this-is-link-to-non-existing-page');
    // should navigate to 404 not found page
    const url = await browser.getCurrentUrl();
    expect(url).to.endWith('404');
  });

  after(async () => {
    await navBarPage.autoSignOut();
  });
});
